@@ -1,11 +1,9 @@
 from __future__ import absolute_import, division, print_function
 
-import warnings
-
+import operator
 from functools import partial, wraps
 from itertools import product, repeat
 from math import factorial, log, ceil
-import operator
 
 import numpy as np
 from toolz import compose, partition_all, get, accumulate, pluck
@@ -14,6 +12,7 @@ from . import chunk
 from .core import _concatenate2, Array, atop, lol_tuples, handle_out
 from .creation import arange
 from .ufunc import sqrt
+from .utils import validate_axis
 from .wrap import zeros, ones
 from .numpy_compat import ma_divide, divide as np_divide
 from ..compatibility import getargspec, builtins
@@ -127,7 +126,7 @@ def reduction(x, chunk, aggregate, axis=None, keepdims=False, dtype=None,
         axis = tuple(range(x.ndim))
     if isinstance(axis, int):
         axis = (axis,)
-    axis = tuple(validate_axis(x.ndim, a) for a in axis)
+    axis = validate_axis(axis, x.ndim)
 
     if dtype is None:
         raise ValueError("Must specify dtype")
@@ -520,34 +519,6 @@ with ignoring(AttributeError):
     nanstd = wraps(chunk.nanstd)(nanstd)
 
 
-def vnorm(a, ord=None, axis=None, dtype=None, keepdims=False, split_every=None,
-          out=None):
-    """ Vector norm
-
-    See np.linalg.norm
-    """
-
-    warnings.warn(
-        "DeprecationWarning: Please use `dask.array.linalg.norm` instead.",
-        UserWarning
-    )
-
-    if ord is None or ord == 'fro':
-        ord = 2
-    if ord == np.inf:
-        return max(abs(a), axis=axis, keepdims=keepdims,
-                   split_every=split_every, out=out)
-    elif ord == -np.inf:
-        return min(abs(a), axis=axis, keepdims=keepdims,
-                   split_every=split_every, out=out)
-    elif ord == 1:
-        return sum(abs(a), axis=axis, dtype=dtype, keepdims=keepdims,
-                   split_every=split_every, out=out)
-    else:
-        return sum(abs(a) ** ord, axis=axis, dtype=dtype, keepdims=keepdims,
-                   split_every=split_every, out=out) ** (1. / ord)
-
-
 def _arg_combine(data, axis, argfunc, keepdims=False):
     """ Merge intermediate results from ``arg_*`` functions"""
     axis = None if len(axis) == data.ndim or data.ndim == 1 else axis[0]
@@ -561,6 +532,7 @@ def _arg_combine(data, axis, argfunc, keepdims=False):
         local_args = argfunc(vals, axis=axis)
         inds = np.ogrid[tuple(map(slice, local_args.shape))]
         inds.insert(axis, local_args)
+        inds = tuple(inds)
         vals = vals[inds]
         arg = arg[inds]
         if keepdims:
@@ -634,10 +606,7 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
         axis = tuple(range(x.ndim))
         ravel = True
     elif isinstance(axis, int):
-        if axis < 0:
-            axis += x.ndim
-        if axis < 0 or axis >= x.ndim:
-            raise ValueError("axis entry is out of bounds")
+        axis = validate_axis(axis, x.ndim)
         axis = (axis,)
         ravel = x.ndim == 1
     else:
@@ -741,7 +710,7 @@ def cumreduction(func, binop, ident, x, axis=None, dtype=None, out=None):
     if dtype is None:
         dtype = getattr(func(np.empty((0,), dtype=x.dtype)), 'dtype', object)
     assert isinstance(axis, int)
-    axis = validate_axis(x.ndim, axis)
+    axis = validate_axis(axis, x.ndim)
 
     m = x.map_blocks(func, axis=axis, dtype=dtype)
 
@@ -797,17 +766,6 @@ def cumprod(x, axis=None, dtype=None, out=None):
     return cumreduction(np.cumprod, _cumprod_merge, 1, x, axis, dtype, out=out)
 
 
-def validate_axis(ndim, axis):
-    """ Validate single axis dimension against number of dimensions """
-    if axis > ndim - 1 or axis < -ndim:
-        raise ValueError("Axis must be between -%d and %d, got %d" %
-                         (ndim, ndim - 1, axis))
-    if axis < 0:
-        return axis + ndim
-    else:
-        return axis
-
-
 def topk(a, k, axis=-1, split_every=None):
     """ Extract the k largest elements from a on the given axis,
     and return them sorted from largest to smallest.
@@ -844,12 +802,7 @@ def topk(a, k, axis=-1, split_every=None):
     >>> d.topk(-2).compute()
     array([1, 3])
     """
-    if isinstance(a, int) and isinstance(k, Array):
-        warnings.warn("DeprecationWarning: topk(k, a) has been replaced with "
-                      "topk(a, k)")
-        a, k = k, a
-
-    axis = validate_axis(a.ndim, axis)
+    axis = validate_axis(axis, a.ndim)
 
     # chunk and combine steps of the reduction, which recursively invoke
     # np.partition to pick the top/bottom k elements from the previous step.
@@ -886,7 +839,7 @@ def argtopk(a, k, axis=-1, split_every=None):
 
     Returns
     -------
-    Selection of int64 indices of x with size abs(k) along the given axis.
+    Selection of np.intp indices of x with size abs(k) along the given axis.
 
     Examples
     --------
@@ -898,10 +851,10 @@ def argtopk(a, k, axis=-1, split_every=None):
     >>> d.argtopk(-2).compute()
     array([1, 2])
     """
-    axis = validate_axis(a.ndim, axis)
+    axis = validate_axis(axis, a.ndim)
 
     # Generate nodes where every chunk is a tuple of (a, original index of a)
-    idx = arange(a.shape[axis], chunks=(a.chunks[axis], ), dtype=np.int64)
+    idx = arange(a.shape[axis], chunks=(a.chunks[axis], ), dtype=np.intp)
     idx = idx[tuple(slice(None) if i == axis else np.newaxis
                     for i in range(a.ndim))]
     a_plus_idx = a.map_blocks(chunk.argtopk_preprocess, idx,
@@ -919,5 +872,5 @@ def argtopk(a, k, axis=-1, split_every=None):
 
     return reduction(
         a_plus_idx, chunk=chunk_combine, combine=chunk_combine,
-        aggregate=aggregate, axis=axis, keepdims=True, dtype=np.int64,
+        aggregate=aggregate, axis=axis, keepdims=True, dtype=np.intp,
         split_every=split_every, concatenate=False, output_size=abs(k))
