@@ -268,6 +268,14 @@ def test_angle():
     assert_eq(da.angle(comp), np.angle(comp))
 
 
+def test_issignedinf():
+    arr = np.random.randint(-1, 2, size=(20, 20)).astype(float) / 0
+    darr = da.from_array(arr, 3)
+
+    assert_eq(np.isneginf(arr), da.isneginf(darr))
+    assert_eq(np.isposinf(arr), da.isposinf(darr))
+
+
 def test_frompyfunc():
     myadd = da.frompyfunc(add, 2, 1)
     np_myadd = np.frompyfunc(add, 2, 1)
